@@ -349,8 +349,8 @@ fn cargo_build_command(
                 build.enable_zig_ar = true;
                 let zig_triple = if target.is_linux() && !target.is_musl_libc() {
                     match context.platform_tag.iter().find(|tag| tag.is_manylinux()) {
-                        Some(PlatformTag::Manylinux { x, y }) => {
-                            format!("{target_triple}.{x}.{y}")
+                        Some(PlatformTag::Manylinux { major, minor }) => {
+                            format!("{target_triple}.{major}.{minor}")
                         }
                         _ => target_triple.to_string(),
                     }
@@ -411,6 +411,11 @@ fn cargo_build_command(
         }
     }
 
+    // Set PYO3_BUILD_EXTENSION_MODULE when building pyo3 extension modules
+    if bridge_model.is_pyo3() && !bridge_model.is_bin() {
+        build_command.env("PYO3_BUILD_EXTENSION_MODULE", "1");
+    }
+
     // Setup `PYO3_CONFIG_FILE` if we are cross compiling for pyo3 bindings
     if let Some(interpreter) = python_interpreter {
         // Target python interpreter isn't runnable when cross compiling
