@@ -23,11 +23,12 @@
 
 #![deny(missing_docs)]
 
-pub use crate::bridge::{Bindings, BridgeModel};
+pub use crate::bridge::{Abi3Version, BridgeModel, PyO3, PyO3Crate};
 pub use crate::build_context::{BuildContext, BuiltWheelMetadata};
-pub use crate::build_options::{BuildOptions, CargoOptions};
+pub use crate::build_options::{BuildOptions, CargoOptions, TargetTriple};
 pub use crate::cargo_toml::CargoToml;
 pub use crate::compile::{compile, BuildArtifact};
+pub use crate::compression::{CompressionMethod, CompressionOptions};
 pub use crate::develop::{develop, DevelopOptions};
 #[cfg(feature = "schemars")]
 pub use crate::generate_json_schema::{generate_json_schema, GenerateJsonSchemaOptions, Mode};
@@ -39,10 +40,11 @@ pub use crate::module_writer::{
 pub use crate::new_project::{init_project, new_project, GenerateProjectOptions};
 pub use crate::pyproject_toml::PyProjectToml;
 pub use crate::python_interpreter::PythonInterpreter;
-pub use crate::target::Target;
+pub use crate::source_distribution::find_path_deps;
 #[cfg(feature = "upload")]
 pub use crate::upload::{upload, upload_ui, PublishOpt, Registry, UploadError};
 pub use auditwheel::PlatformTag;
+pub use target::Target;
 
 mod auditwheel;
 mod bridge;
@@ -53,6 +55,7 @@ mod cargo_toml;
 /// Generate CI configuration
 pub mod ci;
 mod compile;
+mod compression;
 mod cross_compile;
 mod develop;
 mod generate_json_schema;
