@@ -2,11 +2,12 @@ use crate::PyProjectToml;
 use anyhow::{bail, format_err, Context, Result};
 use fs_err as fs;
 use indexmap::IndexMap;
+use normpath::PathExt;
 use pep440_rs::{Version, VersionSpecifiers};
 use pep508_rs::{
     ExtraName, ExtraOperator, MarkerExpression, MarkerTree, MarkerValueExtra, Requirement,
 };
-use pyproject_toml::License;
+use pyproject_toml::{check_pep639_glob, License};
 use regex::Regex;
 use serde::{Deserialize, Serialize};
 use std::collections::{HashMap, HashSet};
@@ -14,6 +15,7 @@ use std::fmt::Write as _;
 use std::path::{Path, PathBuf};
 use std::str;
 use std::str::FromStr;
+use tracing::debug;
 
 /// The metadata required to generate the .dist-info directory
 #[derive(Serialize, Deserialize, Debug, Clone, Eq, PartialEq)]
@@ -53,7 +55,7 @@ pub struct Metadata24 {
     pub maintainer: Option<String>,
     pub maintainer_email: Option<String>,
     pub license: Option<String>,
-    // https://peps.python.org/pep-0639/#license-file-multiple-use
+    pub license_expression: Option<String>,
     pub license_files: Vec<PathBuf>,
     pub classifiers: Vec<String>,
     pub requires_dist: Vec<Requirement>,
@@ -88,6 +90,7 @@ impl Metadata24 {
             maintainer: None,
             maintainer_email: None,
             license: None,
+            license_expression: None,
             license_files: vec![],
             classifiers: vec![],
             requires_dist: vec![],
@@ -145,6 +148,41 @@ impl Metadata24 {
                 bail!("`project.dynamic` must not specify `name` in pyproject.toml");
             }
 
+            // According to PEP 621, build backends must not add metadata fields
+            // that are not declared in the dynamic list. Clear fields from Cargo.toml
+            // that are not in the dynamic list.
+            if !dynamic.contains("description") {
+                self.summary = None;
+            }
+            if !dynamic.contains("authors") {
+                self.author = None;
+                self.author_email = None;
+            }
+            if !dynamic.contains("maintainers") {
+                self.maintainer = None;
+                self.maintainer_email = None;
+            }
+            if !dynamic.contains("keywords") {
+                self.keywords = None;
+            }
+            if !dynamic.contains("urls") {
+                self.project_url.clear();
+            }
+            if !dynamic.contains("license") {
+                self.license = None;
+                // Don't clear license_files as they may come from auto-discovery
+            }
+            if !dynamic.contains("classifiers") {
+                self.classifiers.clear();
+            }
+            if !dynamic.contains("readme") {
+                self.description = None;
+                self.description_content_type = None;
+            }
+            if !dynamic.contains("requires-python") {
+                self.requires_python = None;
+            }
+
             self.name.clone_from(&project.name);
 
             let version_ok = pyproject_toml.warn_invalid_version_info();
@@ -185,9 +223,9 @@ impl Metadata24 {
                     if let Some(readme_path) = file {
                         let readme_path = pyproject_dir.join(readme_path);
                         let description = Some(fs::read_to_string(&readme_path).context(format!(
-                                "Failed to read readme specified in pyproject.toml, which should be at {}",
-                                readme_path.display()
-                            ))?);
+                            "Failed to read readme specified in pyproject.toml, which should be at {}",
+                            readme_path.display()
+                        ))?);
                         self.description = description;
                     }
                     if let Some(description) = text {
@@ -198,42 +236,89 @@ impl Metadata24 {
                 None => {}
             }
 
+            // Make sure existing license files from `Cargo.toml` are relative to the project root
+            let license_files = std::mem::take(&mut self.license_files);
+            for license_file in license_files {
+                let license_path = license_file
+                    .strip_prefix(pyproject_dir)
+                    .with_context(|| {
+                        format!(
+                            "license file `{}` exists outside of the project root `{}`",
+                            license_file.display(),
+                            pyproject_dir.display()
+                        )
+                    })?
+                    .to_path_buf();
+                self.license_files.push(license_path);
+            }
+
             if let Some(requires_python) = &project.requires_python {
                 self.requires_python = Some(requires_python.clone());
             }
 
             if let Some(license) = &project.license {
                 match license {
-                    // TODO: switch to License-Expression core metadata, see https://peps.python.org/pep-0639/#add-license-expression-field
-                    License::Spdx(license_expr) => self.license = Some(license_expr.clone()),
+                    // PEP 639
+                    License::Spdx(license_expr) => {
+                        self.license_expression = Some(license_expr.clone())
+                    }
+                    // Deprecated by PEP 639
                     License::File { file } => {
-                        let license_path = pyproject_dir.join(file);
-                        self.license_files.push(license_path);
+                        self.license_files.push(file.to_path_buf());
                     }
                     License::Text { text } => self.license = Some(text.clone()),
                 }
             }
 
-            // Until PEP 639 is approved with metadata 2.3, we can assume a
-            // dynamic license-files (also awaiting full 2.2 metadata support)
-            // We're already emitting the License-Files metadata without issue.
-            // license-files.globs = ["LICEN[CS]E*", "COPYING*", "NOTICE*", "AUTHORS*"]
-            let license_include_targets = ["LICEN[CS]E*", "COPYING*", "NOTICE*", "AUTHORS*"];
-            let escaped_manifest_string = glob::Pattern::escape(pyproject_dir.to_str().unwrap());
-            let escaped_manifest_path = Path::new(&escaped_manifest_string);
-            for pattern in license_include_targets.iter() {
-                for license_path in
-                    glob::glob(&escaped_manifest_path.join(pattern).to_string_lossy())?
-                        .filter_map(Result::ok)
-                {
-                    if !license_path.is_file() {
-                        continue;
+            // Handle PEP 639 license-files field
+            if let Some(license_files) = &project.license_files {
+                // Safe on Windows and Unix as neither forward nor backwards slashes are escaped.
+                let escaped_pyproject_dir =
+                    PathBuf::from(glob::Pattern::escape(pyproject_dir.to_str().unwrap()));
+                for license_glob in license_files {
+                    check_pep639_glob(license_glob)?;
+                    for license_path in
+                        glob::glob(&escaped_pyproject_dir.join(license_glob).to_string_lossy())?
+                    {
+                        let license_path = license_path?;
+                        if !license_path.is_file() {
+                            continue;
+                        }
+                        let license_path = license_path
+                            .strip_prefix(pyproject_dir)
+                            .expect("matched path starts with glob root")
+                            .to_path_buf();
+                        if !self.license_files.contains(&license_path) {
+                            debug!("Including license file `{}`", license_path.display());
+                            self.license_files.push(license_path);
+                        }
                     }
-                    // if the pyproject.toml specified the license file,
-                    // then we won't list it as automatically included
-                    if !self.license_files.contains(&license_path) {
-                        eprintln!("📦 Including license file \"{}\"", license_path.display());
-                        self.license_files.push(license_path);
+                }
+            } else {
+                // Auto-discovery of license files for backwards compatibility
+                // license-files.globs = ["LICEN[CS]E*", "COPYING*", "NOTICE*", "AUTHORS*"]
+                let license_include_targets = ["LICEN[CS]E*", "COPYING*", "NOTICE*", "AUTHORS*"];
+                let escaped_manifest_string =
+                    glob::Pattern::escape(pyproject_dir.to_str().unwrap());
+                let escaped_manifest_path = Path::new(&escaped_manifest_string);
+                for pattern in license_include_targets.iter() {
+                    for license_path in
+                        glob::glob(&escaped_manifest_path.join(pattern).to_string_lossy())?
+                            .filter_map(Result::ok)
+                    {
+                        if !license_path.is_file() {
+                            continue;
+                        }
+                        let license_path = license_path
+                            .strip_prefix(pyproject_dir)
+                            .expect("matched path starts with glob root")
+                            .to_path_buf();
+                        // if the pyproject.toml specified the license file,
+                        // then we won't list it as automatically included
+                        if !self.license_files.contains(&license_path) {
+                            eprintln!("📦 Including license file `{}`", license_path.display());
+                            self.license_files.push(license_path);
+                        }
                     }
                 }
             }
@@ -399,7 +484,14 @@ impl Metadata24 {
             project_url.insert("Source Code".to_string(), repository.clone());
         }
         let license_files = if let Some(license_file) = package.license_file.as_ref() {
-            vec![manifest_path.as_ref().join(license_file)]
+            let license_path = manifest_path.as_ref().join(license_file).normalize()?;
+            if !license_path.is_file() {
+                bail!(
+                    "license file `{license_file}` specified in `{}` is not a file",
+                    manifest_path.as_ref().display()
+                );
+            }
+            vec![license_path.into_path_buf()]
         } else {
             Vec::new()
         };
@@ -476,7 +568,8 @@ impl Metadata24 {
         let license_files: Vec<String> = self
             .license_files
             .iter()
-            .map(|path| path.file_name().unwrap().to_str().unwrap().to_string())
+            // Use a portable path to ensure the metadata is consistent between Unix and Windows.
+            .map(|path| path.display().to_string().replace("\\", "/"))
             .collect();
         add_vec("License-File", &license_files);
 
@@ -494,6 +587,9 @@ impl Metadata24 {
         add_option("Author-email", &self.author_email);
         add_option("Maintainer", &self.maintainer);
         add_option("Maintainer-email", &self.maintainer_email);
+        // PEP 639
+        add_option("License-Expression", &self.license_expression);
+        // Deprecated by PEP 639
         add_option("License", &self.license.as_deref().map(fold_header));
         add_option(
             "Requires-Python",
@@ -627,6 +723,7 @@ mod test {
     use expect_test::{expect, Expect};
     use indoc::indoc;
     use pretty_assertions::assert_eq;
+    use tempfile::TempDir;
 
     fn assert_metadata_from_cargo_toml(
         readme: &str,
@@ -828,6 +925,23 @@ A test project
         );
     }
 
+    #[test]
+    fn test_pep639() {
+        let manifest_dir = PathBuf::from("test-crates").join("pyo3-mixed");
+        let cargo_metadata = MetadataCommand::new()
+            .manifest_path(manifest_dir.join("Cargo.toml"))
+            .exec()
+            .unwrap();
+        let mut metadata = Metadata24::from_cargo_toml(&manifest_dir, &cargo_metadata).unwrap();
+        let pyproject_toml = PyProjectToml::new(manifest_dir.join("pyproject.toml")).unwrap();
+        metadata
+            .merge_pyproject_toml(&manifest_dir, &pyproject_toml)
+            .unwrap();
+
+        assert_eq!(metadata.license_expression.as_ref().unwrap(), "MIT");
+        assert_eq!(metadata.license.as_ref(), None);
+    }
+
     #[test]
     fn test_merge_metadata_from_pyproject_dynamic_license_test() {
         let manifest_dir = PathBuf::from("test-crates").join("license-test");
@@ -848,12 +962,12 @@ A test project
         assert_eq!(4, metadata.license_files.len());
 
         // Verify pyproject.toml license = {file = ...} worked
-        assert_eq!(metadata.license_files[0], manifest_dir.join("LICENCE.txt"));
+        assert_eq!(metadata.license_files[0], PathBuf::from("LICENCE.txt"));
 
         // Verify the default licenses were included
-        assert_eq!(metadata.license_files[1], manifest_dir.join("LICENSE"));
-        assert_eq!(metadata.license_files[2], manifest_dir.join("NOTICE.md"));
-        assert_eq!(metadata.license_files[3], manifest_dir.join("AUTHORS.txt"));
+        assert_eq!(metadata.license_files[1], PathBuf::from("LICENSE"));
+        assert_eq!(metadata.license_files[2], PathBuf::from("NOTICE.md"));
+        assert_eq!(metadata.license_files[3], PathBuf::from("AUTHORS.txt"));
     }
 
     #[test]
@@ -894,4 +1008,149 @@ A test project
             assert_eq!(result, expected);
         }
     }
+
+    #[test]
+    fn test_issue_2544_respect_pyproject_dynamic_without_dynamic_fields() {
+        let temp_dir = TempDir::new().unwrap();
+        let crate_path = temp_dir.path();
+        let manifest_path = crate_path.join("Cargo.toml");
+        let pyproject_path = crate_path.join("pyproject.toml");
+
+        // Create basic src structure
+        fs::create_dir(crate_path.join("src")).unwrap();
+        fs::write(crate_path.join("src/lib.rs"), "").unwrap();
+
+        // Write Cargo.toml with metadata that should NOT be included
+        // because pyproject.toml doesn't declare them as dynamic
+        let cargo_toml = indoc!(
+            r#"
+            [package]
+            name = "test-package"
+            version = "0.1.0"
+            description = "Description from Cargo.toml - should not appear"
+            authors = ["author from cargo.toml <author@example.com>"]
+            keywords = ["cargo", "toml", "keyword"]
+            repository = "https://github.com/example/repo"
+
+            [lib]
+            crate-type = ["cdylib"]
+            "#
+        );
+        fs::write(&manifest_path, cargo_toml).unwrap();
+
+        // Write pyproject.toml WITHOUT declaring the fields as dynamic
+        let pyproject_toml_content = indoc!(
+            r#"
+            [build-system]
+            requires = ["maturin>=1.0,<2.0"]
+            build-backend = "maturin"
+
+            [project]
+            name = "test-package"
+            version = "0.1.0"
+            # Note: no description, authors, keywords, urls in dynamic list
+            # dynamic = []  # Not specified, so defaults to empty
+            "#
+        );
+        fs::write(&pyproject_path, pyproject_toml_content).unwrap();
+
+        // Load metadata as maturin does
+        let cargo_metadata = MetadataCommand::new()
+            .manifest_path(&manifest_path)
+            .exec()
+            .unwrap();
+        let mut metadata = Metadata24::from_cargo_toml(crate_path, &cargo_metadata).unwrap();
+        let pyproject_toml = PyProjectToml::new(&pyproject_path).unwrap();
+        metadata
+            .merge_pyproject_toml(crate_path, &pyproject_toml)
+            .unwrap();
+
+        assert_eq!(
+            metadata.summary, None,
+            "summary should be None when not in dynamic list"
+        );
+        assert_eq!(
+            metadata.author, None,
+            "author should be None when not in dynamic list"
+        );
+        assert_eq!(
+            metadata.keywords, None,
+            "keywords should be None when not in dynamic list"
+        );
+        assert!(
+            metadata.project_url.is_empty(),
+            "project_url should be empty when not in dynamic list"
+        );
+    }
+
+    #[test]
+    fn test_issue_2544_respect_pyproject_dynamic_with_dynamic_fields() {
+        let temp_dir = TempDir::new().unwrap();
+        let crate_path = temp_dir.path();
+        let manifest_path = crate_path.join("Cargo.toml");
+        let pyproject_path = crate_path.join("pyproject.toml");
+
+        // Create basic src structure
+        fs::create_dir(crate_path.join("src")).unwrap();
+        fs::write(crate_path.join("src/lib.rs"), "").unwrap();
+
+        // Write Cargo.toml with metadata
+        let cargo_toml = indoc!(
+            r#"
+            [package]
+            name = "test-package"
+            version = "0.1.0"
+            description = "Description from Cargo.toml - should appear"
+            authors = ["author from cargo.toml <author@example.com>"]
+            keywords = ["cargo", "toml", "keyword"]
+            repository = "https://github.com/example/repo"
+
+            [lib]
+            crate-type = ["cdylib"]
+            "#
+        );
+        fs::write(&manifest_path, cargo_toml).unwrap();
+
+        // Write pyproject.toml WITH fields declared as dynamic
+        let pyproject_toml_content = indoc!(
+            r#"
+            [build-system]
+            requires = ["maturin>=1.0,<2.0"]
+            build-backend = "maturin"
+
+            [project]
+            name = "test-package"
+            version = "0.1.0"
+            # Fields declared as dynamic - should come from Cargo.toml
+            dynamic = ["description", "authors", "keywords", "urls"]
+            "#
+        );
+        fs::write(&pyproject_path, pyproject_toml_content).unwrap();
+
+        // Load metadata as maturin does
+        let cargo_metadata = MetadataCommand::new()
+            .manifest_path(&manifest_path)
+            .exec()
+            .unwrap();
+        let mut metadata = Metadata24::from_cargo_toml(crate_path, &cargo_metadata).unwrap();
+        let pyproject_toml = PyProjectToml::new(&pyproject_path).unwrap();
+        metadata
+            .merge_pyproject_toml(crate_path, &pyproject_toml)
+            .unwrap();
+
+        // These fields SHOULD be set because they are in dynamic list
+        assert_eq!(
+            metadata.summary,
+            Some("Description from Cargo.toml - should appear".to_string())
+        );
+        assert_eq!(
+            metadata.author,
+            Some("author from cargo.toml <author@example.com>".to_string())
+        );
+        assert_eq!(metadata.keywords, Some("cargo,toml,keyword".to_string()));
+        assert_eq!(
+            metadata.project_url.get("Source Code"),
+            Some(&"https://github.com/example/repo".to_string())
+        );
+    }
 }
