@@ -223,6 +223,13 @@ pub struct ToolMaturin {
     pub unstable_flags: Option<Vec<String>>,
     /// Additional rustc arguments
     pub rustc_args: Option<Vec<String>>,
+    /// Use base Python executable instead of venv Python executable in PEP 517 build.
+    //
+    // This can help avoid unnecessary rebuilds, as the Python executable does not change
+    // every time. It should not be set when the sdist build requires packages installed
+    // in venv.
+    #[serde(default)]
+    pub use_base_python: bool,
 }
 
 /// A pyproject.toml as specified in PEP 517
@@ -379,10 +386,7 @@ impl PyProjectToml {
                     if !version_specifier.contains(&self_version) {
                         eprintln!(
                             "⚠️  Warning: You specified {requires_maturin} in pyproject.toml under \
-                            `build-system.requires`, but the current {maturin} version is {version}",
-                            requires_maturin = requires_maturin,
-                            maturin = maturin,
-                            version = self_version,
+                            `build-system.requires`, but the current {maturin} version is {self_version}",
                         );
                         return false;
                     }
