@@ -219,7 +219,7 @@ fn develop_hello_world(#[case] backend: TestInstallBackend, #[case] name: &str)
     handle_result(develop::test_develop(
         "test-crates/hello-world",
         None,
-        format!("develop-hello-world-{}", name).as_str(),
+        format!("develop-hello-world-{name}").as_str(),
         false,
         backend,
     ));
@@ -246,7 +246,7 @@ fn develop_pyo3_ffi_pure(#[case] backend: TestInstallBackend, #[case] name: &str
     handle_result(develop::test_develop(
         "test-crates/pyo3-ffi-pure",
         None,
-        format!("develop-pyo3-ffi-pure-{}", name).as_str(),
+        format!("develop-pyo3-ffi-pure-{name}").as_str(),
         false,
         backend,
     ));
@@ -580,6 +580,16 @@ fn warn_on_missing_python_source() {
     handle_result(errors::warn_on_missing_python_source())
 }
 
+#[test]
+fn pypi_compatibility_unsupported_target() {
+    handle_result(errors::pypi_compatibility_unsupported_target())
+}
+
+#[test]
+fn pypi_compatibility_mixed_tags() {
+    handle_result(errors::pypi_compatibility_mixed_tags())
+}
+
 #[test]
 #[cfg_attr(not(target_os = "linux"), ignore)]
 fn musl() {
@@ -638,6 +648,7 @@ fn workspace_members_non_local_dep_sdist() {
         edition = "2021"
         description = "Implements a dummy function (get_fortytwo.DummyClass.get_42()) in rust"
         license = "MIT"
+        readme = "README.md"
 
         [dependencies]
         pyo3 = { version = "0.25.0", features = [
@@ -925,6 +936,30 @@ fn workspace_inheritance_sdist() {
     ))
 }
 
+#[test]
+fn workspace_license_files() {
+    handle_result(other::test_source_distribution(
+        "test-crates/hello-world",
+        SdistGenerator::Cargo,
+        expect![[r#"
+            {
+                "hello_world-0.1.0/Cargo.lock",
+                "hello_world-0.1.0/Cargo.toml",
+                "hello_world-0.1.0/LICENSE",
+                "hello_world-0.1.0/PKG-INFO",
+                "hello_world-0.1.0/README.md",
+                "hello_world-0.1.0/check_installed/check_installed.py",
+                "hello_world-0.1.0/licenses/AUTHORS.txt",
+                "hello_world-0.1.0/pyproject.toml",
+                "hello_world-0.1.0/src/bin/foo.rs",
+                "hello_world-0.1.0/src/main.rs",
+            }
+        "#]],
+        None,
+        "sdist-hello-world",
+    ))
+}
+
 #[test]
 fn abi3_python_interpreter_args() {
     handle_result(other::abi3_python_interpreter_args());
